@@ -23,7 +23,9 @@
 #include <math.h>
 #include <getopt.h>
 
-#define VERSION "1.0"
+#define VERSION "1.2"
+#define max(A,B) ( (A) > (B) ? (A):(B))
+#define min(A,B) ( (A) > (B) ? (B):(A))
 
 struct option longopts[] = {
   {"help", no_argument, NULL, 'h'},
@@ -36,6 +38,8 @@ struct option longopts[] = {
   {"depth", required_argument, NULL, 'd'},
   {"level", required_argument, NULL, 'l'},
   {"eat", required_argument, NULL, 'e'},
+  {"noeatmore", no_argument, NULL, 'n'},
+  {"noeatall", no_argument, NULL, 'a'},
   {"single", no_argument, NULL, 's'},
   {"hint", no_argument, NULL, 'i'},
   {"xawale", no_argument, NULL, 'x'},
@@ -44,12 +48,13 @@ struct option longopts[] = {
 
 static char *progname;
 int flagh = 0, flagv = 0, flagr = 0, flagb = 0, flago = 0, flagc = 0, flagp = 0;
-int flagd = 0, flagl = 0, flage = 0, flags = 0, flagi = 0, flagx = 0, lose = 0;
-int depth = 9;                     // initial depth for the min-max algorithm
+int flagd = 0, flagl = 0, flage = 0, flagn = 0, flaga = 0, flags = 0, flagi = 0;
+int flagx = 0, lose = 0;
+int depth = 9;                    // initial depth for the min-max algorithm
 int level = 0;                    // level of game
 int neh = 4;                      // max number of eaten holes
 int computer = 1, human = 0;
-double tmax[3] = {5, 20, 100};
+double tmax[3] = {2, 20, 100};
 
 int
 main (int argc, char *argv[])
@@ -70,7 +75,7 @@ main (int argc, char *argv[])
   progname = argv[0];
 
   while ((optc =
-	  getopt_long (argc, argv, "hvrb:o:c:p:d:l:e:six", longopts,
+	  getopt_long (argc, argv, "hvrb:o:c:p:d:l:e:nasix", longopts,
 		       (int *) 0)) != -1)
     switch (optc)
       {
@@ -150,6 +155,12 @@ main (int argc, char *argv[])
 	  }
 	level--;
 	break;
+      case 'n':
+	flagn = 1;
+	break;
+      case 'a':
+        flaga = 1;
+        break;
       case 's':
 	flags = 1;
 	break;
@@ -178,7 +189,7 @@ main (int argc, char *argv[])
 /* help */
   if (flagh)
     {
-      printf ("awale, the african game.\n\n");
+      printf ("free awale.\n\n");
       printf ("Usage: %s [OPTION] \n\n", progname);
       printf ("\
   -h, --help      display this help and exit\n\
@@ -197,6 +208,8 @@ main (int argc, char *argv[])
   -d N, --depth=N  initial depth N of the algorithm (default N=9)\n\
   -l L, --level=L level L=1, 2 or 3 of game (default L=1)\n\
   -e E, --eat=E   maximum number of holes which can be eaten (default E=4)\n\
+  -n, --noeatmore eat nothing when trying to eat more than E holes\n\
+  -a, --noeatall  eat nothing when trying to eat all remaining stones\n\
   -s, --single    process a single move\n\
   -i, --hint      print hints\n\
   -x  --xawale    start in engine mode. A single line output is provided at\n\
@@ -213,7 +226,7 @@ main (int argc, char *argv[])
 /* version */
   if (flagv)
     {
-      printf ("awale %s\n", VERSION);
+      printf ("free awale %s\n", VERSION);
       printf ("Written by Alain and Laurent Le Bot, Copyright (C) 2006\n\n\
 There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A\n\
 PARTICULAR PURPOSE.  You may redistribute copies of awale under the terms\n\
@@ -363,7 +376,6 @@ int
 best_move (int hole[], int score[], char *label)
 {
   int valid (int *, int);
-  int max (int, int);
   int delay (int *, int *, int);
   int value (int *, int *, int, int);
   int distribute (int *, int *, int, char *);
@@ -530,19 +542,20 @@ print_board (int hole[], int score[], int move, char *label)
 int
 distribute (int hole[], int score[], int i, char *label)
 {
-  int player;
   int end_game (int *, int *, int);
-  int who_play (int);
-  int n, j, jj, nt;
+  int who_play (int), who_wait (int);
+  int player, other;
+  int n, i0, i1, j, jj, j0, j1, nt;
 
   player = who_play (i);
+  other = who_wait (i);
 
 /* distribution of stones */
 
   n = hole[i];
   hole[i] = 0;
-  j = 0;
 
+  j = 0;
   do
     {
       j++;
@@ -558,14 +571,37 @@ distribute (int hole[], int score[], int i, char *label)
 
 /* score calculation */
 
+  j1=jj;                // j1 is the last hole reached
   nt = 1;
-  while (jj / 6 != player && (hole[jj] == 2 || hole[jj] == 3) && nt <= neh)
+  while (jj / 6 != player && (hole[jj] == 2 || hole[jj] == 3))
     {
-      score[player] = score[player] + hole[jj];
-      hole[jj] = 0;
       jj = (jj - 1 + 12) % 12;
       nt++;
     }
+  nt--;                 // nt is the number of eatable holes 
+
+  if (flagn == 0 || nt <= neh)
+    {
+
+     j0=j1-min(nt,neh); // eaten holes run from j0+1 to j1
+
+     if (flaga) 
+       {
+        for (i0=other*6, i1=i0+6, jj=i0; jj<i1; jj++) 
+          {
+           if ((jj<=j0 || jj>j1) && hole[jj]!=0) break;
+          }
+       }                // all stones of the opponent are eaten when jj=i1
+
+     if (!flaga || jj!=i1)
+       {
+        for (jj=j1; jj>j0; jj--)
+          {
+           score[player] = score[player] + hole[jj];
+           hole[jj] = 0;
+          }             // score is updated
+       }
+    }
 
   if (end_game (hole, score, i))
     {
@@ -702,7 +738,6 @@ value (int hole2[], int score2[], int i, int p)
   int valid (int *, int);
   int who_play (int), who_wait (int);
   int distribute (int *, int *, int, char *);
-  int max (int, int);
 
   int hole[12];
   int score[2];
@@ -745,7 +780,6 @@ delay (int hole2[], int score2[], int i)
 {
   int who_play (int), who_wait (int);
   int distribute (int *, int *, int, char *);
-  int max (int, int);
   int hole[12];
   int score[2];
   char label[4] = "RUN";
@@ -775,15 +809,3 @@ delay (int hole2[], int score2[], int i)
 
 }
 
-/*cccccccccccccccccccccccccccccccccccccccccccccccccccccccc
-! Return the max of a and b.
-!ccccccccccccccccccccccccccccccccccccccccccccccccccccccc*/
-
-int
-max (int a, int b)
-{
-  if (a > b)
-    return a;
-  else
-    return b;
-}
